
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef EKGPLOTTER_H
#define EKGPLOTTER_H

#include <iostream>
#include <string>
#include <time.h>
#include "EKGbuffer.h"
#include "EKGtools.h"
#include "calibrate.h"
using namespace std;


/**
 * Ausgabe des EKG als Postscript-Datei oder in Rohdaten.
 * Die zu schreibenden Daten werden in einem EKGbuffer dem Konstruktor
 * bergeben. Name kann der Patientenname (und das Geburtsdatum) bergeben
 * werden. Mit toPostscript() und/oder toRaw() wird in einen ostreram
 * geschreiben. Die Zeit- und Spannungsskalen fr die Postscriptausgabe sind
 * mit 50 mm/s und 20 mm/mV voreingestellt und knnen mit ts und vs
 * gendert werden.
 * @author Lars Stollenwerk, Nov. 2002
 */
class EKGplotter
{
  public:
    /// Konstruktor. b enthlt die zu zeichnenden Daten.
    EKGplotter( const EKGbuffer & b ) : ts(50), vs(20), B(b) {}
    /** 
     * Ausgabe als Postscript. 
     * @param o stream to write to
     */
    void toPostscript( ostream & o );
    /** 
     * Ausgabe der Mewerte. Die werte sind duch TAB getrennt. Der Header
     * ist mit # auskommentiert und enthlt Patienteninformationen,
     * Datum und Spaltenbeschreibung.
     * @param o stream to write to
     */
    void toRaw( ostream & o );
    /// Zeitskala in mm/s 
    float ts;
    /// Spannungsskala in mm/mV
    float vs;
    /// Name des Patienten
    string name;
    /**
     * Anzahl der Zeitschritte auf einem Blatt ermitteln.
     * Lnge des Papiers = 265 mm -> max. Lnge der Kurve in Zeitschritten.
     * @param TS Zeitskala
     * @return Anzahl der Zeitschritte.
     */
    static int timesteps_on_paper( float TS ) 
      { return static_cast<int>(265 / TS / EKG_TimeStep); }
  private:
    /// Kopie der Daten. 
    const EKGbuffer B;
    /** 
     * Unterfunktion von toPostscript. Es werden ab Position n alle
     * nichtleeren Strings aus s nach o geschrieben. n zeigt danach auf die
     * leere Zeile.
     */
    static void writestrings( const char ** const s, int & i, ostream & o );
    /**
     * Daten in ostream schreiben. Schreibt Kurve n aus Puffer b nach o und
     * benutzt Skalierung aus ts und vs.
     */
    void writedata( EKGbuffer b, int n, ostream & o );
    // Schreibt aVR aus b in o
    void writeavr( EKGbuffer b, ostream & o );
    // Schreibt aVL aus b in o
    void writeavl( EKGbuffer b, ostream & o );
    // Schreibt aVF aus b in o
    void writeavf( EKGbuffer b, ostream & o );
};

#endif

