
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003, 2011, 2015 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef GEKGDISPLAY_H
#define GEKGDISPLAY_H

#include <gtkmm.h> 
#include <unistd.h>
#include "calibrate.h"
#include "EKGdevice.h"
#include "EKGbuffer.h"
#include "EKGtools.h"
using namespace std;


/**
 * Diese Klasse stellt ein laufendes EKG graphisch dar. Die dargestellte
 * Zeit ist proportional zur Breite. Die Hhe beeinflusst den y-Mastab.
 */
class gEKGdisplay : public EKGdevice, public Gtk::DrawingArea
{
  public:
    /** Konstruktor */
    gEKGdisplay();
    /** Destruktor */
    ~gEKGdisplay();
    /** 
     * berschreiben; wird bein realisieren und jeder Grennderung
     * aufgerufen.
     */
    virtual bool on_configure_event(GdkEventConfigure* p0);
    /** 
     * bershrieben zum Neuzeichnen des Widgets.
     */
    virtual bool on_expose_event(GdkEventExpose* p0);
    /**
     * berschreiben aus EKGdevice. Eingehende Werte werden angenommen, zum
     * Zeichnen vorgemerkt und an das nchste Gert weitergegebe.
     */
    virtual void input( float * d );
    /**
     * Lscht alle Werte und setzt Schreibposition auf den rechten Rand.
     */
    void reset();
    /** 
     * Gibt eine Referenz auf die Wertetabelle des Displays zurck.
     */
    const EKGbuffer & getValues() const { return Values; }
  private:
    // ----------------------------------------------------------------------
    // Wertetabellen
    // ----------------------------------------------------------------------
    /**
     * Dieser EKGbuffer enthlt alle Werte, die vom Display dargestellt
     * werden. 
     */
    EKGbuffer Values;
    // ----------------------------------------------------------------------
    // Darstellung
    // ----------------------------------------------------------------------
    /**
     * Zeiger auf letzte Schreibposition im Display, also auf den jngsten
     * Wert.
     * <code>
     * |----->   >-----|
     *       ^ Plast
     * </code>
     */
    int Plast;
    /**
     * Breite der Lcke zwischen altem und neuem Graphen in Pixeln.
     * <code>
     * |----->   >-----|
     *        ^^^ GapWidth
     * </code>
     * Es gilt: Values.size() + GapWidth = width()
     */
    static const int GapWidth = 20;
    /**
     * Minimale Breite in Pixeln. Wird das Display kleiner gestellt, so
     * findet keine Darstellung mehr statt.
     */
    static const int MinWidth = 100;
    /* 
     * Null-Linie
     */
    Gdk::Color ZLcolor;
    /** 
     * Hintergrundfarbe 
     */
    Gdk::Color BGcolor;
    /** 
     * Zeichenfarbe 
     */
    Gdk::Color FGcolor;
    /**
     * Zum Widget gehrendes Gdk-Window. Wird von configure_event_impl()
     * gesetzt.
     */
    Glib::RefPtr< Gdk::Window > win;
    /**
     * Echtzeit Aktualisierung. Diese Metode zeichnet neu eingegangene
     * Werte und speichdert sie in Values. Sie wird vom Gtk::Main::input
     * event aufgerufen, wenn es was zu zeichnen gibt.
     * @param fd beobachteter filedeskriptor, also Pipe[0].
     */
    bool drawRealtime( Glib::IOCondition );
    // ----------------------------------------------------------------------
    // Saklierung
    // ----------------------------------------------------------------------
    /**
     * Positionen der Kurven am Bildschirm. Diese Werte geben die
     * y-Koordinaten der Null-Linien an. Sie werden von
     * configure_event_impl() gesetzt.
     */
    gint zeroPos[6];
    /**
     * Mittlere Eingangsempfindlichkeit der Meverstrker. Hier wird die
     * maximale Eingangsspannung eines mittleren Meverstrkers in mV
     * gespeichert.  Sie wird von scaleY() bentigt und vom Konstruktor
     * gesetzt.
     * <code> YScaleFactor = < EKG_amp > * 255 </code><br>
     */
    float mVmax;
    /**
     * Rechnet die y-Position eines Wertes aus.
     * @param y y-Werte
     * @param n Nummer der Kurve, beginnend mit 0.
     * @return y-Position in Pixeln
     */
    gint scaleY( float y, int n );
    /**
     * Rechnet die y-Werte fr alle sechs Kurven aus. Es werden die zur
     * Erweiterung auf 6 Kurven ntigen aufrufe von aVX durchgefhrt.
     * @param y Wertetripel.
     * @param Y Ausgabe: Werte 6-tupel
     */
    void scaleY( float * y, gint * Y );
    /**
     * Rechtsverschiebung mit berlauf. Gibt a + b zurck, wobei hinter
     * dem rechten Rand (width()) bei 0 weitergezhlt wird.
     */
    gint addX( gint a, gint b );
    // ----------------------------------------------------------------------
    // Thread Kommunikation
    // ----------------------------------------------------------------------
    /**
     * Pipe zur Threadkommunikation. ber diesen Pipe teilt input() mit,
     * da drawRealtime() aufgerufen werden soll. Pipe[0] ist zum lesen,
     * Pipe[1] ist zum schreiben.
     */
    int Pipe[2];
    /** 
     * Gre eines Datenkrbchens. Wird bentigt, um auf pipe zu lesen und
     * zu schreiben.
     */
    static const int dsize = 3 * sizeof(float);
};


// ----------------------------------------------------------------------
// inline Deklarationen
// ----------------------------------------------------------------------

inline gint gEKGdisplay::addX( gint a, gint b )
{
  gint c = a + b;
  if ( c >= get_width() ) 
    c -= get_width();
  return c;
};


/* Rechnet einen Punkt fr Kurve i in Bildschirmposition um */
inline gint gEKGdisplay::scaleY( float y, int n  )
{
    return static_cast<gint>( -y / mVmax * get_height() / 6 ) + zeroPos[n] ; 
};
#endif

